import React, { useEffect, useState } from 'react';
import { animated, config, useSpring } from '@react-spring/web';
import { UiFunction, useMasonryColumns, useMedia } from './utilities';
import * as s from './styles';
export const Backdrop = ({ children, isVisible = false }) => {
    // TODO: Hide overflow on body
    // TODO: Prevent scrolling
    // TODO: Prevent clicking, typing etc. in forms
    const styles = useSpring({
        config: config.stiff,
        opacity: isVisible ? 1 : 0,
        zIndex: isVisible ? 2000 : -1,
        ...s.Backdrop,
    });
    return React.createElement(animated.div, { style: styles }, children);
};
export const Btn = ({ children, isFullWidth = false, uiFunction = UiFunction.None, ...rest }) => {
    const [baseStyle, setBaseStyle] = useState(s.Btn);
    const [finalStyle, setFinalStyle] = useState(s.Btn);
    const [focusStyle, setFocusStyle] = useState(s.Btn);
    const [hoverStyle, setHoverStyle] = useState(s.Btn);
    const [isHovering, setIsHovering] = useState(false);
    const [isFocused, setIsFocused] = useState(false);
    // TODO: create remaining UI Function styles and implement here
    useEffect(() => {
        switch (uiFunction) {
            case UiFunction.Light:
                setBaseStyle(s.BtnLight);
                setFocusStyle(s.BtnLightFocus);
                setHoverStyle(s.BtnLightHover);
                setFinalStyle(s.BtnLight);
                break;
            case UiFunction.Secondary:
                setBaseStyle(s.BtnSecondary);
                setFocusStyle(s.BtnSecondaryFocus);
                setHoverStyle(s.BtnSecondaryHover);
                setFinalStyle(s.BtnSecondary);
                break;
            case UiFunction.Success:
                setBaseStyle(s.BtnSuccess);
                setFocusStyle(s.BtnSuccessFocus);
                setHoverStyle(s.BtnSuccessHover);
                setFinalStyle(s.BtnSuccess);
                break;
            default:
                setBaseStyle(s.BtnPrimary);
                setFocusStyle(s.BtnPrimaryFocus);
                setHoverStyle(s.BtnPrimaryHover);
                setFinalStyle(s.BtnPrimary);
                break;
        }
    }, [uiFunction]);
    useEffect(() => {
        if (isFocused) {
            setFinalStyle(focusStyle);
            return;
        }
        if (isHovering) {
            setFinalStyle(hoverStyle);
            return;
        }
        setFinalStyle(baseStyle);
    }, [isFocused, isHovering, baseStyle]);
    return (React.createElement("button", { style: { ...finalStyle, width: (isFullWidth) ? '100%' : 'auto' }, ...rest, onMouseEnter: () => { setIsHovering(true); }, onMouseLeave: () => { setIsHovering(false); }, onFocus: () => { setIsFocused(true); }, onBlur: () => { setIsFocused(false); } }, children));
};
export const Card = ({ children }) => {
    return React.createElement("div", { style: s.Card }, children);
};
export const CardBody = ({ children }) => {
    return React.createElement("div", { style: s.CardBody }, children);
};
export const CardText = ({ children }) => {
    return React.createElement("div", { style: s.CardText }, children);
};
export const CardTitle = ({ children }) => {
    return React.createElement("h5", { style: s.CardTitle }, children);
};
export const CloseButton = ({ ...rest }) => {
    const [finalStyle, setFinalStyle] = useState(s.CloseButton);
    const [isHovering, setIsHovering] = useState(false);
    useEffect(() => {
        const newStyle = {
            ...s.CloseButton,
            opacity: (isHovering) ? '.5' : 1
        };
        setFinalStyle(newStyle);
    }, [isHovering]);
    return (React.createElement("button", { style: finalStyle, ...rest, onMouseEnter: () => { setIsHovering(true); }, onMouseLeave: () => { setIsHovering(false); } }));
};
export const Dropdown = ({ children }) => {
    return (React.createElement("div", { style: s.Dropdown }, children));
};
export const DropdownMenu = ({ children, isExpanded = false }) => {
    return (React.createElement("div", { style: { ...s.DropdownMenu, display: (isExpanded) ? 'block' : 'none' } }, children));
};
export const FloatingTextBox = ({ helpText, label, name, value, ...rest }) => {
    const textboxId = name + "Textbox";
    const [inputStyle, setInputStyle] = useState(s.FormFloatingFormControl);
    const [labelStyle, setLabelStyle] = useState(s.FloatingLabel);
    useEffect(() => {
        if (value) {
            setLabelStyle(s.FloatingLabelFocus);
        }
    }, []);
    const thisFocus = (e) => {
        setInputStyle({
            ...s.FormControlFocus,
            ...s.FormFloatingFormControl,
        });
        setLabelStyle(s.FloatingLabelFocus);
    };
    const thisBlur = (e) => {
        if (!e.target.value || e.target.value.length === 0) {
            setLabelStyle(s.FloatingLabel);
        }
        setInputStyle(s.FormFloatingFormControl);
    };
    return (React.createElement(React.Fragment, null,
        React.createElement("div", { style: s.FormFloating },
            React.createElement("input", { style: inputStyle, id: textboxId, name: name, onFocus: (e) => { e.target.select(); thisFocus(e); }, onBlur: (e) => { thisBlur(e); }, value: value, ...rest }),
            React.createElement("label", { style: labelStyle, htmlFor: textboxId }, label)),
        (helpText && helpText.length > 0)
            ? React.createElement("div", { style: { fontSize: '.875em', marginTop: '0.25rem', opacity: '0.7' } }, helpText)
            : null));
};
export const HamburgerIcon = () => {
    return (React.createElement("span", { style: s.NavBarTogglerIcon }));
};
export const Masonry = ({ children }) => {
    const columnCount = useMasonryColumns();
    const [columns, setColumns] = useState([]);
    useEffect(() => {
        const newCols = [];
        const colWidth = (100 / columnCount) + '%';
        const colStyle = {
            ...s.MasonryCol,
            width: colWidth,
        };
        if (children) {
            const childArray = children;
            for (let i = 0; i < columnCount; i++) {
                const rows = [];
                for (let j = i; j < childArray.length; j += columnCount) {
                    rows.push(React.createElement("div", { key: j, style: s.MasonryBrick }, childArray[j]));
                }
                newCols.push(React.createElement("div", { key: i, style: colStyle }, rows));
            }
            setColumns(newCols);
        }
        else {
            setColumns([]);
        }
    }, [children, columnCount]);
    return (React.createElement("div", { style: s.Masonry }, columns));
};
export const ModalBody = ({ children }) => {
    return (React.createElement("div", { style: s.ModalBody }, children));
};
export const ModalContent = ({ children }) => {
    return (React.createElement("div", { style: s.ModalContent }, children));
};
export const ModalDialog = ({ children }) => {
    // TODO: Sizing doesn't go to full width when below 576px
    const [style, setStyle] = useState(s.ModalDialog);
    const width = useMedia(['(min-width: 576px)'], [1], 0);
    useEffect(() => {
        if (width === 0)
            setStyle({
                ...s.ModalDialog,
                width: '100%',
                margin: '.75rem'
            });
        else
            setStyle({
                ...s.ModalDialog,
                width: '500px'
            });
    }, [width]);
    return (React.createElement("div", { style: style }, children));
};
export const ModalFooter = ({ children }) => {
    return (React.createElement("div", { style: s.ModalFooter }, children));
};
export const ModalHeader = ({ children }) => {
    return (React.createElement("div", { style: s.ModalHeader }, children));
};
export const NavBar = ({ children, isFixedTop = false }) => {
    let finalStyle = s.NavBar;
    if (isFixedTop && isFixedTop === true)
        finalStyle = {
            ...s.NavBar,
            ...s.FixedTop,
            backgroundColor: '#f8f9fa'
        };
    return (React.createElement("nav", { style: finalStyle }, children));
};
export const NavBarToggler = ({ isVisible = true, ...rest }) => {
    const [finalStyle, setFinalStyle] = useState(s.NavBarToggler);
    if (isVisible === true)
        return (React.createElement("button", { type: 'button', style: finalStyle, ...rest, onFocus: () => { setFinalStyle(s.NavBarTogglerFocus); }, onBlur: () => { setFinalStyle(s.NavBarToggler); } },
            React.createElement("span", { style: s.NavBarTogglerIcon })));
    return null;
};
export const NavBarCollapse = ({ children, isVertical = false, isVisible = true }) => {
    const [finalStyle, setFinalStyle] = useState(s.NavBarCollapseVer);
    useEffect(() => {
        let newStyle = (isVertical === true)
            ? s.NavBarCollapseVer
            : s.NavBarCollapseHor;
        if (isVisible === false)
            newStyle = {
                ...newStyle,
                display: 'none'
            };
        setFinalStyle(newStyle);
    }, [isVertical, isVisible]);
    return (React.createElement("div", { style: { ...finalStyle } }, children));
};
export const NavBarNav = ({ children, isVertical = false }) => {
    const [finalStyle, setFinalStyle] = useState(s.NavBarNavVer);
    useEffect(() => {
        if (isVertical === true)
            setFinalStyle(s.NavBarNavVer);
        else
            setFinalStyle(s.NavBarNavHor);
    }, [isVertical]);
    return (React.createElement("div", { style: finalStyle }, children));
};
export const ScrollToTop = ({ isVisible = false }) => {
    const styles = useSpring({
        config: config.stiff,
        opacity: isVisible ? 1 : 0,
        ...s.ScrollToTop,
    });
    return React.createElement(animated.div, { style: styles },
        React.createElement(Btn, { title: 'Top', onClick: () => { window.scrollTo({ top: 0, behavior: 'smooth' }); }, uiFunction: UiFunction.Primary, type: 'button' }, "\u2912"));
};
export const Stringify = ({ o }) => {
    return (React.createElement("pre", null, JSON.stringify(o, null, 2)));
};
export const Switch = ({ caption, id, onBlur, onChange, onFocus, ...rest }) => {
    // TODO: focused states
    const [currentState, setCurrentState] = useState(s.FormCheckInput);
    const thisChanged = (e) => {
        if (e.target.checked)
            setCurrentState(s.FormCheckInputChecked);
        else
            setCurrentState(s.FormCheckInput);
        if (onChange)
            onChange(e);
    };
    return (React.createElement("div", { style: s.FormCheckSwitch },
        React.createElement("input", { onChange: (e) => { thisChanged(e); }, style: currentState, type: "checkbox", role: "switch", id: id, ...rest }),
        (caption && caption.length > 0)
            ? React.createElement("label", { htmlFor: id }, caption)
            : null));
};
